@@ -115,3 +115,8 @@ wchar_t* bctbx_string_to_wide_string(const char* str){
 	bctbx_error("Conversion from string to wide string is not implemented");
     return NULL;
 }
+
+unsigned int bctbx_get_code_page(const char* encoding){
+	bctbx_error("Getting code page is not implemented");
+	return 0;
+}
