@@ -380,6 +380,7 @@ void merge_and_print_results_files(void) {
 				bc_tester_printf(bc_printf_verbosity_error, "Empty suite results file '%s'", file_name);
 			}
 			remove(file_name);
+			bctbx_file_close(bctbx_file);
 		} else {
 			bc_tester_printf(bc_printf_verbosity_error, "Failed to open suite results file '%s'", file_name);
 			//Assume suite crash and report it.
@@ -851,14 +852,18 @@ int bc_tester_run_parallel(void) {
 				print_timer = bctbx_get_cur_time_ms();
 			}
 			elapsed = bctbx_get_cur_time_ms();
-		} while (testsFinished < nb_test_suites && elapsed < timeout);
+		} while (testsFinished < nb_test_suites);
+// This code has been commented out in order to display failing test of timed out suites in the allure report
+//		} while (testsFinished < nb_test_suites && elapsed < timeout);
 
 		if (elapsed >= timeout) {
 			bc_tester_printf(bc_printf_verbosity_error, "Stopped waiting for all test suites to execute as we reach timeout. Killing running suites.");
-			for(int i = 0 ;  i < nextSuite ; ++i){
+			// This code has been commented out in order to display failing test of timed out suites in the allure report
+/*			for(int i = 0 ;  i < nextSuite ; ++i){
 				if( processed[i] == 0)
 					TerminateProcess(suitesPids[i].hProcess, -1);
 			}
+*/
 			bc_tester_printf(bc_printf_verbosity_error, "*** Test suite took too much time. Please check errors or split longest test suites to benefit from parallel execution. ***");
 			ret = -1;
 		}
@@ -942,12 +947,15 @@ int bc_tester_run_parallel(void) {
 			print_timer = bctbx_get_cur_time_ms();
 		}
 		elapsed = bctbx_get_cur_time_ms();
-	} while (testsFinished < nb_test_suites && elapsed < timeout);
+	} while (testsFinished < nb_test_suites);
+// This code has been commented out in order to display failing test of timed out suites in the allure report
+//	} while (testsFinished < nb_test_suites && elapsed < timeout);
 
 	if (elapsed >= timeout) {
 		bc_tester_printf(bc_printf_verbosity_error, "Stopped waiting for all test suites to execute as we reach timeout. Killing running suites.");
 		bc_tester_printf(bc_printf_verbosity_error, "*** Test suite took too much time. Please check errors or split longest test suites to benefit from parallel execution. ***");
-		kill_sub_processes(suitesPids);
+		// This code has been commented out in order to display failing test of timed out suites in the allure report
+		//kill_sub_processes(suitesPids);
 		ret = -1;
 	}
 	bc_tester_printf(bc_printf_verbosity_info, "All suites ended.");
@@ -1348,9 +1356,11 @@ int bc_tester_parse_args(int argc, char **argv, int argid)
 		max_vm_kb = atol(argv[i]);
 	} else if (strcmp(argv[i], "--resource-dir") == 0) {
 		CHECK_ARG("--resource-dir", ++i, argc);
+		if (bc_tester_resource_dir_prefix) bctbx_free(bc_tester_resource_dir_prefix);
 		bc_tester_resource_dir_prefix = strdup(argv[i]);
 	} else if (strcmp(argv[i], "--writable-dir") == 0) {
 		CHECK_ARG("--writable-dir", ++i, argc);
+		if (bc_tester_writable_dir_prefix) bctbx_free(bc_tester_writable_dir_prefix);
 		bc_tester_writable_dir_prefix = strdup(argv[i]);
 	}else if(strcmp(argv[i],"--child")==0){//Switch off this parameter as it is used for external processing
 	} else {
