@@ -19,6 +19,8 @@
 
 #ifdef __APPLE__
 
+#include <TargetConditionals.h>
+
 #import <Foundation/Foundation.h>
 #include <dlfcn.h>
 
@@ -27,6 +29,11 @@
 #include "bctoolbox/logging.h"
 #include "ios_utils_stub.hh"
 
+#if TARGET_OS_IPHONE
+#import <UIKit/UIKit.h>
+#endif
+
+
 namespace bctoolbox {
 
 std::unique_ptr<IOSUtils> IOSUtils::sInstance = nullptr;
@@ -104,6 +111,16 @@ bool IOSUtils::isApplicationStateActive() {
     return mUtils->isApplicationStateActive();
 }
 
+int IOSUtils::getOSMajorVersion() const{
+#if TARGET_OS_IPHONE
+    NSArray *versionCompatibility = [[UIDevice currentDevice].systemVersion componentsSeparatedByString:@"."];
+	return [[versionCompatibility objectAtIndex:0] intValue];
+#else
+	bctbx_error("IOSUtils::getOSMajorVersion() not running on iOS");
+	return 0;
+#endif
+}
+
 
 unsigned long IOSUtilsStub::beginBackgroundTask(const char *name, std::function<void()> cb) {
     return 0;
