@@ -38,6 +38,9 @@ extern test_suite_t utils_test_suite;
 extern test_suite_t crypto_test_suite;
 extern test_suite_t parser_test_suite;
 extern test_suite_t ios_utils_test_suite;
+extern test_suite_t encrypted_vfs_test_suite;
+extern test_suite_t param_string_test_suite;
+extern test_suite_t vfs_test_suite;
 
 #ifdef __cplusplus
 };
